package comprti.cback;
import java.lang.reflect.*;
import java.util.*;
import comprti.msg.*;
import comprti.*;
/** 
* @see 
* @author H. Timucin Ozdemir
* @date August 6, 1999 (Updated on August 7, 1999)
*/
public class DCallbackInvoker extends CallbackInvoker {
  Class    tclass = null;  
  String   methodName = null;
  Object[] Args = null;
  Method meth = null;
  public DCallbackInvoker(MessageHandler target, String _methodName, Object[] _Args) 
         throws IllegalArgumentException {
    super(target);
    tclass=target.getClass();
    if(_methodName==null)  
       throw new IllegalArgumentException(" Illegal Method name : "+_methodName);     
    methodName=_methodName; Args=_Args;
    try{ meth=getMethod(tclass,methodName,Args,target==null);
         System.out.println("DCallbackInvoker: Method "+methodName+" is found!");
    } catch(NoSuchMethodException nsme) {  
       System.out.println("WARNING: Method "+methodName+" is not found. Using default!");
       meth=null;
      }
  }// end of CONSTRUCTOR

  public void invoke(Message inMsg) throws IllegalArgumentException {
    boolean executeDefault=(meth==null);
    if( !(inMsg instanceof UpdateAttributeMessage) && !(inMsg instanceof UpdateParameterMessage) )
       throw new IllegalArgumentException("DCallbackInvoker.invoke(): Message type is not Update!");
    if(!executeDefault) {
      Object ret = null;
      Object[] args=null;
      if(inMsg instanceof UpdateAttributeMessage) {
         UpdateAttributeMessage uMsg = (UpdateAttributeMessage)inMsg;
         args = new Object[2];
         args[0] = new Integer(uMsg.getObjectId());
         args[1] = uMsg.getValue();
      }
      if(inMsg instanceof UpdateParameterMessage) {
         args=new Object[1];
         args[0]=((UpdateParameterMessage)inMsg).getValue();
      }
      if(args!=null) 
        try   { ret = meth.invoke(getTarget(),args); } 
        catch ( InvocationTargetException e) 
        { System.out.println("InvocationTargetException: "+methodName); executeDefault=true; } 
        catch ( IllegalAccessException e)    
        { System.out.println("IllegalAccessException: "+methodName); executeDefault=true; }
      else executeDefault=true;
    }// end of if
    if(executeDefault) super.invoke(inMsg); 
  }// end of invoke()

  private static Method getMethod(Class theClass, String methodName, 
    Object[] args, boolean useStatic) throws NoSuchMethodException {
    Method[] meths = theClass.getMethods();
    int numberOfArguments = 0;
    if (args != null) {
      numberOfArguments = args.length;
    }
    for(int i=0;i<meths.length;i++) {
      if (meths[i].getName().equals(methodName) &&
          meths[i].getParameterTypes().length == numberOfArguments &&
          (!useStatic || (meths[i].getModifiers() & Modifier.STATIC) != 0)) {
        // weve got the right number of arguments
        // check to see if they are equal or assignable
        boolean match = true;
        Class[] types = meths[i].getParameterTypes();
        for(int j=0;j<numberOfArguments && match;j++) {
          match = match && 
             (args[j] == null ||
              types[j].equals(args[j].getClass()) ||
              types[j].isAssignableFrom(args[j].getClass()));
        }
        if (match) {
          return meths[i];
        } else {
        }
      }
    }
    throw new NoSuchMethodException("cannot find method " + 
      methodName + " in class " + theClass);

  }// end of getMethod()

} // end of class DCallbackInvoker 
