package comprti.queue;

import java.util.*;

/**
* Implements the Event Queue object. This queue has one tail and
* multiple heads for each consumer. Each consumer also implements
* Notifier interface so that queue only informs them through this
* interface and it is compumer proxy object's reponsibility to pop
* the data from the queue.
*
* @author Zeynep Odcikin-Ozdemir
* @date   De 20, 1998
*
*/
public class 
EventQueueImpl 
{
   /** Head */
   QItem          gHead;

   /** Tail */
   QItem          tail;

   /** Registered Listeners */
   Vector         regis ;

   /** Notification Adminstrator */
   NotifierAdmin  nAdmin;

   /** Collection of head pointers belonging to each proxy consumer. */
   Hashtable      heads;

   /**
   */
   public EventQueueImpl()
   { 
      regis = new Vector(); 
      nAdmin = new NotifierAdmin();
      gHead = tail = new QItem(null);
      heads = new Hashtable();
 
   }// end of CONSTRUCTOR()

   /**
   *
   * Registers an object which is interested in the message arrival
   * to the event channel.
   *
   * @param The notifier interface reference.
   */
   public void register(Notifier a)  
   { 
      if( !regis.contains(a) ) { regis.addElement(a); heads.put(a,tail); }

   }// end of register()

   /**
   *
   * Deregisters the listener.
   *
   * @param The notifier interface reference.
   * @exception worb.protocols.iiop.es.EvtUtils.UnknownUser 
   *
   */
   public void deregister(Notifier a) 
    throws UnknownUser 
   {
      if( !regis.contains(a) ) throw new UnknownUser();
      else { regis.removeElement(a); heads.remove(a); }
 
   }// end of deregister()

   /**
   * Adds a data to the event queue.
   *
   * @param The data to be added to the queue.
   * @param The notifier reference which adds the data.
   * @exception worb.protocols.iiop.es.EvtUtils.NotMember
   *
   */
   public void 
   push(org.omg.CORBA.Any data, Notifier a)
        throws NotMember
   {
      System.out.println(" in push() of EQ"+regis.size());
      QItem b = new QItem(null);
      tail.set(data,regis.size());
      tail.next(b);
      tail = b;
      nAdmin.broadcast(regis.elements());
   }// end of push()

   /**
   * 
   * Extracts the data from the queue.
   * 
   * @param The notifier interface reference which pops up the data.
   * @exception worb.protocols.iiop.es.EvtUtils.NotMember
   * @exception worb.protocols.iiop.es.EvtUtils.QueueIsEmpty
   *
   */
   public org.omg.CORBA.Any pop(Notifier a) throws NotMember, QueueIsEmpty
   {
      org.omg.CORBA.Any ret = null;

      // If it is not registered, throw exception.
      if( !regis.contains(a) ) throw new NotMember();
      
      // Get the head pointer for this consumer
      QItem  myHead = (QItem) heads.get(a);

      // If it exists
      if( myHead!=null )
      {
         // If it is the last element, throw
         if( myHead==tail)
            throw new QueueIsEmpty();
         
         // Decrement the number consumers
         myHead.decrement();

         // Gets the data
         ret = (org.omg.CORBA.Any) myHead.getObject();
          
         // Deallocates the elements from the queue if necessary.
         if(myHead.count()==0) cleanUp();
         myHead = myHead.next();
         heads.remove(a);
         heads.put(a,myHead);
         
      } else throw new QueueIsEmpty();

      return ret;

   }// end of pop()

   /**
   */
   void cleanUp()
   {
      while( gHead.count()==0 && gHead!=tail ) gHead=gHead.next();

   }// end of cleanUp()
 
}// end of EventQueue

