package comprti.queue;
import java.util.*;

/**
* Administers the Notifier threads.
*
* @author H. Timucin Ozdemir
* @date   August 7, 1999 (Updated on August 7,1999)
*/
public class NotifierAdmin {
  /** Head of the Message List */
  ListItem   head; 

  /** Head of the Free Notifier Threads List */
  ListItem   freeNotifiers;

  /** Head of the Bussy NotifierThread List */
  ListItem   bussyNotifiers;
 
  /** */
  public NotifierAdmin() {
     for(int i=0;i<2;i++) {
        NotifierThread a = new NotifierThread(this);
        addFreeList(a);
     }// end of for
  }// end of constructor

  /**
  * Broadcasts the availability of the message to the given 
  * listeners.
  * @param not_set a set of notifier interfaces.
  */
  public void broadcast( Enumeration not_set ) {
     System.out.println(" broadcast() of NotifierAdmin ");
     if( not_set.hasMoreElements() ) {
       //NotifierThread worker = new NotifierThread();
       //worker.broadcast(not_set); 
       // If there is an available NotifierThread 
       if( freeNotifiers!=null ) {
          ListItem a = freeNotifiers;
          freeNotifiers = freeNotifiers.next();
          ((NotifierThread)a.getObject()).broadcast( not_set );
          a.prev(null);
          a.next(bussyNotifiers);
          if(bussyNotifiers!=null) bussyNotifiers.prev(a);
          bussyNotifiers = a;
       } else
         {  
            System.out.println(" Broadcast Message is added to the List");
            // There is no NotifierThread, then add this to message queue.
            ListItem a1 = new ListItem( not_set );
            a1.next(head);
            if(head!=null) head.prev(a1);
            head = a1; 
         }
     } else System.out.println("There is no listener!");

  }// end of broadcast()

  /** 
  * A NotifierThread object informs the administrator that
  * it finished the informing process. Administrator looks at the
  * message queue, if there is some mesages in the queue, then
  * the first one is asigned to this available NotifierThread.
  *
  * @param a A reference to the NotifierThread object
  */
  public synchronized void setFree( NotifierThread a ) {
     boolean found = false;
     ListItem current = bussyNotifiers;

     // Find the record related to this NotifierThread.
     while( !found && current!=null ) {
        if( current.getObject()==a) found = true;
        else current = current.next();
     }// end of while

     if( found ) {
        // If there is a message to broadcast
        if( head!=null ) {
           ListItem a1 = head;
           head = head.next();
           a.broadcast((Enumeration)a1.getObject());
        }
        else
         {
            if( current==bussyNotifiers) bussyNotifiers=current.next();
            if( current.prev()!=null ) current.prev().next(current.next());
            if( current.next()!=null ) current.next().prev(current.prev());
            current.next(freeNotifiers);
            current.prev(null);
            if( freeNotifiers!=null ) freeNotifiers.prev(current);
            freeNotifiers = current;
            notifyAll();

         }

     }// end of if

  }// end of setFree()
  
  /**
  * @param a NotifierThread object.
  */
  public void addFreeList( NotifierThread a ) {
     ListItem b = new ListItem(a);
     b.next(freeNotifiers);
     if(freeNotifiers!=null) freeNotifiers.prev(b);
     freeNotifiers = b;
  }// end of addFreeList()

  /** */
  public synchronized void shut_down( ) {
     while( head!=null || bussyNotifiers!=null ) {         
      try {  wait(); } catch (InterruptedException ie) { continue; }
      System.out.println("shut_down");
     }// end of while()
     ListItem b = freeNotifiers;
     while( b!=null ) 
     { ((NotifierThread)b.getObject()).stop( ); b=b.next(); }
  }// end of shut_down()

  public void dispose( ) { shut_down(); }

  public static void
  main(String[] args) {
    int i;
    NotifierAdmin na = new NotifierAdmin();
    Vector a = new Vector();
    for(i=0;i<3;i++) a.addElement(new NotifierImpl("aa_"+i));
    for(i=0;i<3;i++) na.broadcast(a.elements());
    na.dispose();
    System.out.println("End of shut_down");
  }// end of TestStub

}// end of NotifierAdmin()

