package comprti.queue;
import java.util.*;
import comprti.*;
import comprti.msg.*;
import comprti.cback.*;
import xml.*;
/** 
* @see PullProxy, Notifier, Queue, CallbackRepository, MessageHandler
* @author  H. Timucin Ozdemir 
*/
public class PullProxyImpl implements PullProxy, Runnable  {
  RTIComponent source=null;
  FedPubSubInfo filter;
  CallbackRepository cbackRep=null;
  PullQueue que;
  Thread runner;
  boolean running=false; 
  public PullProxyImpl( RTIComponent _source, CallbackRepository _cbRep, 
                        PullQueue _que, FedPubSubInfo _filter ) { 
     source=_source; cbackRep=_cbRep; que=_que; filter=_filter;
     if(que!=null) { que.registerPullProxy((PullProxy)this); start(); } 
  }// end of CONSTRUCTOR
  public PullProxyImpl(CallbackRepository _cbackRep, FedPubSubInfo _filter) 
  { this(null,_cbackRep,null,_filter); }
  public PullProxyImpl(PullQueue _queue, FedPubSubInfo _filter) 
  { this(null,null,_queue,_filter); }
  public void setCallbackRepository(CallbackRepository _cbackRep) 
  { cbackRep=_cbackRep;} 
  public void setRTIComponent(RTIComponent _source) 
  { source=_source;}
  public void setQueue(PullQueue _queue) { 
    if(que!=null) { if(running) stop(); que.deregisterPullProxy((PullProxy)this); } 
    que=_queue; 
    if(que!=null) { que.registerPullProxy((PullProxy)this); if(!running) start(); } 
  }// end of setQueue()
  public void start() 
  { if( runner==null ) runner = new Thread(this); runner.start(); }
  public void stop() 
  {  running=false; if( runner!=null ) { runner.stop(); runner=null; } }
  public synchronized void run() {
    running = true;
    while( running ) {
      while( !query() ) { 
        System.out.println("PullProxyImpl:: before wait()!");
        try { wait(); } catch(InterruptedException ie) { continue; }
      }
      try { 
        System.out.println("PullProxyImpl:: I wake up!");
        Message aMsg = que.getMessage(this); 
        if(aMsg!=null&&(aMsg.getRTIComponent()==null||aMsg.getRTIComponent()!=source))
           dispatch(aMsg);
      } catch(NotMember nm) {}
        catch(QueueIsEmpty qie) {}
    }// end of while
  }// end of run()
  boolean query() {
    boolean ret=false;
    try { ret=que.canI(this); }
    catch(NotMember nm) { System.out.println("PullProxyImpl:: NotMember!"); ret=false;}
    return ret;
  }
  // PullProxy&Notifier interface 
  public synchronized void available() { 
    System.out.println("PullProxyImpl:: available!"); 
    if(running) notifyAll();
    else System.out.println("PullProxyImpl:: not running!");
  }// end of available()
  void dispatch(Message aMsg) { 
    CallbackInvoker cInvoker = null;
          if( aMsg instanceof UpdateAttributeMessage ) {
             UpdateAttributeMessage uam = (UpdateAttributeMessage)aMsg; 
             if(interested(uam))   
                cInvoker=cbackRep.getCallbackForAttribute(uam.getObjectName(), 
                                                          uam.getAttributeName());

    } else if( aMsg instanceof UpdateParameterMessage )  {
            UpdateParameterMessage upm = (UpdateParameterMessage)aMsg; 
            if(interested(upm))   
             cInvoker=cbackRep.getCallbackForParameter(upm.getInteractionClassName(),
                                                        upm.getParameterName());
    } else if(interested(aMsg))  cInvoker = cbackRep.getDefaultCallback();
    if(cInvoker!=null) cInvoker.invoke(aMsg);
  }// end of dispatch()
  boolean interested(UpdateAttributeMessage uam) {
    //System.out.println("PullProxyImpl:: interested(UpdateAttributeMessage uam) !"); 
    return ( filter==null || 
            ( filter.canConsumeObjectType(uam.getObjectName()) && 
              filter.canConsumeAttribute(uam.getObjectName(),
                                         uam.getAttributeName()) ) ); 
  }// end of interested()
  boolean interested(UpdateParameterMessage upm) {
    //System.out.println("PullProxyImpl:: interested(UpdateParameterMessage upm) !"); 
    return ( filter==null || 
            ( filter.canConsumeInteractionType(upm.getInteractionClassName()) && 
              filter.canConsumeParameter(upm.getInteractionClassName(),
                                         upm.getParameterName()) ) ); 
  }// end of interested()
  boolean interested(Message aMsg) {
    //System.out.println("PullProxyImpl:: interested(Message aMsg) !"); 
    boolean ret=false;
         if(aMsg instanceof UpdateAttributeMessage) 
            ret=interested((UpdateAttributeMessage)aMsg);
    else if(aMsg instanceof UpdateParameterMessage) 
            ret=interested((UpdateParameterMessage)aMsg);
    return ret;
  }// end of interested()
}// end of PullProxyImpl 
