package comprti.queue;
import java.util.*;
import comprti.msg.*;
import comprti.*;

/** 
* @see Message, MessagePublisher, PullProxy,  PullProxyRegistry, Queue
* @author H. Timucin Ozdemir
* @date August 6, 1999 (Updated on August 6, 1999)
*/
public class 
PushSupPullConQueue implements PullQueue, MessagePublisher {

  /** Head */
  QItem          gHead;
  /** Tail */
  QItem          tail;
  /** Registered Listeners */
  Vector         regis ;
  /** Notification Adminstrator */
  NotifierAdmin  nAdmin;
  /** Collection of head pointers belonging to each proxy consumer. */
  Hashtable      heads;

  public PushSupPullConQueue() { 
    regis = new Vector(); 
    nAdmin = new NotifierAdmin();
    gHead = tail = new QItem(null);
    heads = new Hashtable();
  }// end of CONSTRUCTOR()

  // MessagePublisher Interface
  public void push(Message inMsg) {
    System.out.println(" in push() of EQ"+regis.size());
    QItem b = new QItem(null);
    tail.set(inMsg,regis.size()); tail.next(b); tail = b;
    nAdmin.broadcast(regis.elements());
  }// end of push()

  // PullProxyRegistry Interface
  public void registerPullProxy(PullProxy aPullProxy) { 
    if( !regis.contains(aPullProxy) ) 
    { regis.addElement(aPullProxy); heads.put(aPullProxy,tail); }
  }// end of registerPullProxy()
  public void deregisterPullProxy(PullProxy aPullProxy) throws UnknownUser {
      if( !regis.contains(aPullProxy) ) throw new UnknownUser();
      else { regis.removeElement(aPullProxy); heads.remove(aPullProxy); }
  }// end of deregisterPullProxy()

  // Queue Interface
  public void    addMessage(Message inMsg) { push(inMsg); }

  public boolean canI(PullProxy aPullProxy) throws NotMember {
    if( !regis.contains(aPullProxy) ) throw new NotMember();
    return ( heads.get(aPullProxy)!=tail ) ;
  }// end of canI

  public Message 
  getMessage(PullProxy aPullProxy) throws NotMember, QueueIsEmpty {
      Message ret = null;
      // If it is not registered, throw exception.
      if( !regis.contains(aPullProxy) ) throw new NotMember();
      // Get the head pointer for this consumer
      QItem  myHead = (QItem) heads.get(aPullProxy);

      // If it exists
      if( myHead!=null ) {
         if( myHead==tail) throw new QueueIsEmpty(); // If it is the last element, throw
         myHead.decrement(); // Decrement the number consumers
         ret = (Message) myHead.getObject(); // Gets the data

         // Deallocates the elements from the queue if necessary.
         if(myHead.count()==0) cleanUp();
         myHead = myHead.next();
         heads.remove(aPullProxy);
         heads.put(aPullProxy,myHead);
      } else throw new QueueIsEmpty();
      return ret;
   }// end of getMessage()

   void cleanUp() { while(gHead.count()==0&&gHead!=tail) gHead=gHead.next(); }

   public void dispose() {
    if(nAdmin!=null) nAdmin.dispose();
   }// end of dispose()

}// end of PushSupPullConQueue
