
package comprti.queue;

/**
* Implementation for Queue item. Each queue keeps also the
* number of references for this item. It is used by Event Queue
* implementation.
*
* @author Zeynep Odcikin-Ozdemir
* @date   Dec 20, 1998
*
*/
public class QItem 
{
  /** A next item */ 
  QItem  nxt;

  /** a previous item */
  QItem  prv;

  /** an object in the item */
  Object value;

  /** the number of refernces for the object */
  int    count=0;
  
  /**
  * @param _value a reference to the object.
  */
  public QItem(Object _value)
  {  this(_value,0);
  }// end of constructor
 
  /**
  * @param _value a reference to the object.
  * @param _count the number of references for the object.
  */
  public QItem(Object _value, int _count)
  {
     nxt = prv = null;
     value = _value;
     count=(_count>0?_count:0);
  }// end of constructor
 
  /**
  * Returns the next Item.
  * @return The next QueueItem.
  */
  public QItem next() { return nxt; }
  
  /** 
  * Returns the previous Item.
  * @return The previous QueueItem.
  */
  public QItem prev() { return prv; }

  /** 
  * Returns the object kept in the QueueItem.
  * @return an object in the QueueItem.
  */
  public Object getObject() { return value; }

  /**
  * Sets the next item.
  * @param _nxt a reference to the next QueueItem.
  */
  public void next(QItem _nxt) { nxt=_nxt; }
  
  /** 
  * Sets the previous item.
  * @param _prv a reference to the previous QueueItem.
  */
  public void prev(QItem _prv) { prv=_prv; }

  /** 
  * Returns the number of references to this item.
  * @return the number of references to the object.
  */
  public int count() { return count; }

  /**
  * Decrements the count.
  */
  public void decrement() { if(count>0) count--; }

  /**
  * Sets the object and the reference count.
  * @param _value An object.
  * @param _count The number of references for this object.
  */
  public void set(Object _value, int _count)
  {
    value = _value;
    count = _count;
    if(count<0) count=0;

  }// end of set()

}// end of ListItem()

