package lrti;
import RTIcap.*;             // The package containing our stubs.
import npacrti.*;            // The package containing our stubs.
import lrti.rtidb.*;
import comprti.msg.*;
import comprti.*;
/**
*
* This is callback object to receive RTI messages.
*
* <p>
* <ol>
* <p> <li> discoverObjectInstance
* <p> <li> removeObjectInstance[With Time]
* <p> <li> reflectAttributeValues[WithTime]
* <p> <li> provideAttributeValueUpdate
* <p> <li> turnUpdatesOnForObjectInstance
* <p> <li> turnUpdatesOffForObjectInstance
* <p> <li> timeAdcanceGrant
* <p> <li> timeRegulationEnabled
* <p> <li> timeConstrainedEnabled
* </ol>
*
* <p>
* @date August 11, 1999 ( Updated August 11, 1999 )
* @author H. Timucin Ozdemir
*
*/
public class AggregatedFedAmb extends BaseFedAmb {
    String fedName;
    MessageFactory msgFactory;
    MessagePublisher mpub;

    public AggregatedFedAmb( String _fedName) {  
      fedName=_fedName;
    }// end of CONSTRUCTOR

    /*
    public WBFedAmb( RTIObjectsContainer father) {  
        boss = father; 
        try
        { myLog = new PrintWriter(new FileWriter("WBFedAmb"+father.toString()+".log"),true);
        } catch(IOException ioe) {}
    }// end of CONSTRUCTOR
    */

    RTIObjectClassAttributeLookUp objectClassDB;
    String getObjectClassName(short theObjectClass) {
     return objectClassDB.getObjectClassName(theObjectClass);
    }// end of getObjectClassName()

    String getAttributeName(String objectClassName,int attributeId) {
     return objectClassDB.getAttributeName(objectClassName,attributeId);
    }// end of getObjectClassName()

    RTIInteractionLookUp interactionClassDB;
    String getInteractionClassName(short theClass) {
     return interactionClassDB.getInteractionClassName(theClass);
    }// end of getObjectClassName()

    String getParameterName(String interClsName, int paramId) {
     return interactionClassDB.getParameterName(interClsName,paramId);
    }// end of getParameterName()

    ObjectInstanceMapper objectMap;
    int getLocalInstanceId(int theObject, short objClsType) {
     return objectMap.registerRemoteObject(theObject,objClsType);
    }// end of getLocalInstanceId()
    int getLocalInstanceId(int theObject) {
     short objClsType = (short)objectMap.getRemoteInstanceType(theObject); 
     return (objClsType>=0?objectMap.registerRemoteObject(theObject,objClsType):-1);
    }// end of getLocalInstanceId()

    String getObjectTypeOfInstance(int theObject) {
     return null; //objectMap.registerRemoteObject(theObject);
    }// end of getObjectTypeOfInstance()

    void pushAttributeUpdates(String objClsName,int localid, 
                              RTIcap.AttributeHandleValuePair[] theAttributes) {
     String aName;
     for(int i=0;i<theAttributes.length;i++)
      if((aName=getAttributeName(objClsName,theAttributes[i].handle))!=null)
        mpub.push(msgFactory.createUpdateAttributeMessage(objClsName,
                              localid,aName,theAttributes[i].value));
    }// end of pushAttributeUpdates()

    void pushParameterUpdates(String interClsName, 
                              RTIcap.ParameterHandleValuePair[] theParameters) {
     String paramName;
     for(int i=0;i<theParameters.length;i++)
      if((paramName=getParameterName(interClsName,theParameters[i].handle))!=null)
        mpub.push(msgFactory.createUpdateParamaterMessage(interClsName,
                              paramName,theParameters[i].value));
    }// end of pushParameterUpdates()

    /**
    * Add this new discovered object to the EntityManager.
    */
    public synchronized void 
    discoverObjectInstance(int theObject, short theObjectClass)
        throws RTIcap.CouldNotDiscover, RTIcap.ObjectClassNotKnown, 
               RTIcap.InvalidFederationTime, RTIcap.FederateInternalError
    { String objClsName = getObjectClassName(theObjectClass);
      int    localid    = getLocalInstanceId(theObject,theObjectClass);
      if( objClsName!=null && localid>=0 ) 
        mpub.push(msgFactory.createDiscoverObjectMessage(objClsName,localid));
    }// end of discoverObjectInstance()

    /**
    * Update attributes.
    */
    public synchronized void 
    reflectAttributeValuesWithTime(int theObject, 
               RTIcap.AttributeHandleValuePair[] theAttributes, double theTime, 
               String theTag, RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateOwnsAttributes, RTIcap.InvalidFederationTime, 
               RTIcap.FederateInternalError
    { String objClsName = getObjectTypeOfInstance(theObject);
      int    localid    = getLocalInstanceId(theObject);
      if( objClsName!=null && localid>=0 ) 
          pushAttributeUpdates(objClsName,localid,theAttributes);
    }// end of reflectAttributeValuesWithTime()

    /**
    *
    * Find the objects in the EntityManager and updates their received
    * attributes. Note that, CDR provided us the necessary decode/encode
    * mechanism for various data types.
    *
    */
    public void 
    reflectAttributeValues(int theObject, 
               RTIcap.AttributeHandleValuePair[] theAttributes, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.FederateOwnsAttributes, RTIcap.FederateInternalError
    { String objClsName = getObjectTypeOfInstance(theObject);
      int    localid    = getLocalInstanceId(theObject);
      if( objClsName!=null && localid>=0 ) 
          pushAttributeUpdates(objClsName,localid,theAttributes);
    }// end of reflectAttributeValues()

    /**
    */
    public synchronized void 
    receiveInteractionWithTime(short theInteraction, 
               RTIcap.ParameterHandleValuePair[] theParameters, double theTime, 
               String theTag, RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.InteractionClassNotKnown, 
               RTIcap.InteractionParameterNotKnown, 
               RTIcap.InvalidFederationTime, RTIcap.FederateInternalError
    { receiveInteraction(theInteraction,theParameters,theTag); }

    /**
    */
    public synchronized void 
    receiveInteraction(short theInteraction, 
               RTIcap.ParameterHandleValuePair[] theParameters, String theTag)
        throws RTIcap.InteractionClassNotKnown, 
               RTIcap.InteractionParameterNotKnown, RTIcap.FederateInternalError
    { String interClsName = getInteractionClassName(theInteraction);
      if(interClsName!=null) pushParameterUpdates(interClsName,theParameters);
    }// end of receiveInteraction()

    /**
    * Remove the corresponding object from EntityManager.
    */
    public synchronized void 
    removeObjectInstanceWithTime(int theObject, double theTime, String theTag, 
               RTIcap.EventRetractionHandle theHandle)
        throws RTIcap.ObjectNotKnown, RTIcap.InvalidFederationTime, 
               RTIcap.FederateInternalError
    { //if(boss!=null) boss.removeObjectInstance(theObject);
    }// end of removeObjectInstanceWithTime()

    /**
    * Remove the corresponding object from EntityManager.
    */
    public synchronized void 
    removeObjectInstance(int theObject, String theTag)
        throws RTIcap.ObjectNotKnown, RTIcap.FederateInternalError
    { //if(boss!=null) boss.removeObjectInstance(theObject);
    }// end of removeObjectInstanceWithTime()

    /**
    * Ask the corresponding SimCountry object to update its attribute.
    */
    public synchronized void 
    provideAttributeValueUpdate(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotKnown, 
               RTIcap.AttributeNotOwned, RTIcap.FederateInternalError
    { /*myLog.println(" in  provideAttributeValueUpdate("+theObject+")");
      if(  boss!=null ) {
        RTIObject c = boss.getObjectInstance(theObject);
        if( c != null ) c.reflectAttributes(theAttributes);
        else if(WBDEBUG.DEBUG1) System.out.println(" ***I do not know this object ("+theObject+")***");
      } else if(WBDEBUG.DEBUG1) System.out.println(" EntityManager is NULL ");
      */
    }// end of provideAttributeValueUpdate()

    /**
    * Tell simulated object to send its updates to RTI.
    */
    public synchronized void 
    turnUpdatesOnForObjectInstance(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotOwned, 
               RTIcap.FederateInternalError
    { /*myLog.println(" in  turnUpdatesOnForObjectInstance("+theObject+")");
      if( boss!=null ) {
        RTIObject c = boss.getObjectInstance(theObject);
        if( c != null ) c.reflectAttributes(theAttributes);
        else if(WBDEBUG.DEBUG1) System.out.println(" ***I do not know this object ("+theObject+")***");
      } else if(WBDEBUG.DEBUG1) System.out.println(" EntityManager is NULL ");
      */

    }// end of turnUpdatesOnForObjectInstance()

    /**
    * Tell simulated object *NOT* to send its updates to RTI.
    */
    public synchronized void 
    turnUpdatesOffForObjectInstance(int theObject, short[] theAttributes)
        throws RTIcap.ObjectNotKnown, RTIcap.AttributeNotOwned, 
               RTIcap.FederateInternalError
    { 
        /*
        myLog.println(" in  turnUpdatesOffForObjectInstance("+theObject+")");
        if( boss!=null )
        {
          //ChatBudy c = entManager.getEntity(theObject);
          //if( c != null && c instanceof SimChatBudy ) 
          //  ((SimChatBudy)c).UpdateControls(false,theAttributes); 
        } else if(WBDEBUG.DEBUG1) System.out.println(" EntityManager is NULL ");
        */ 

    }// end of turnUpdatesOffForObjectInstance()

    /**
    * Change the federation's time. And inform that time is
    * advanced.
    */
    public synchronized void 
    timeAdvanceGrant(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.TimeAdvanceWasNotInProgress, 
               RTIcap.FederateInternalError
    { 
        //boss.setCurrentTime( theTime );
        //boss.setTimeAdvanceGranted();

    }// end of timeAdvanceGrant()

    /**
    * Change the federate's time. And inform that time is
    * advanced.
    */
    public synchronized void 
    timeRegulationEnabled(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.EnableTimeRegulationWasNotPending, 
               RTIcap.FederateInternalError
    { 
        //boss.setCurrentTime( theTime );
        //boss.setTimeAdvanceGranted();

    }// end of timeRegulationEnabled()

    /**
    * Change the federate's time. And inform that time is
    * advanced.
    */
    public synchronized void 
    timeConstrainedEnabled(double theTime)
        throws RTIcap.InvalidFederationTime, 
               RTIcap.EnableTimeConstrainedWasNotPending, 
               RTIcap.FederateInternalError
    { 
        //boss.setCurrentTime( theTime );
        //boss.setTimeAdvanceGranted();

    }// end of timeConstrainedEnabled()

} // end of class AggregatedFedAmb

