package lrti;

import java.util.*;

import comprti.*;
import comprti.queue.*;
import comprti.cback.*;
import comprti.msg.*;
import lrti.*;
import lrti.rtidb.*;
import lrti.rtidb.inmem.*;
import xml.*;
import util.*;

import RTIcap.*;
import npacrti.*;

/** 
* contains RTIambassador, AggregatedFedAmb, RTIContext, RTILookUps
* @see 
* @author 
*/
public class FederationDefinition implements RTIContextRegistry {
  String federationName;
  int    federationId;
  boolean DEBUG=true;

  RTIambassador       rtiAmb;
  AggregatedFedAmb    fedAmb;
  PushSupPullConQueue discoverObjectMsg;
  PushSupPullConQueue updateAttributetMsg;
  PushSupPullConQueue updateParameterMsg;
  MessageFactory msgFactory;
  RTIInteractionLookUp          interactionDB; 
  RTIObjectClassAttributeLookUp objectClassDB;
  public FederationDefinition( String _federationName,int _federationId,
                               RTIambassador _rtiAmb, AggregatedFedAmb _fedAmb) 
         throws IllegalArgumentException {
    if(_rtiAmb==null || _fedAmb==null) 
       throw new IllegalArgumentException("FederationDefinition::RTIambassador or AggregatedFedAmb is NULL!");
    rtiAmb=_rtiAmb; fedAmb=_fedAmb;
    discoverObjectMsg = new PushSupPullConQueue();
    updateAttributetMsg = new PushSupPullConQueue();
    updateParameterMsg = new PushSupPullConQueue();
    msgFactory = new MessageFactory();
    interactionDB=new InMemRTIInteractionLookUp();
    objectClassDB=new InMemRTIObjectClassAttributeLookUp();
    federationName=_federationName;
  }// end of if

  public RTIambassador    getRTIambassador()    { return rtiAmb; }
  public AggregatedFedAmb getAggregatedFedAmb() { return fedAmb; }

  // RTIContextFactory rtiCtxFactory;
  // ObjectInstanceMapper

  /**
  * @return (objectClassName,objectClassId)
  */
  public Hashtable getRTIObjectClassIds(Vector in) {
    Hashtable ret = null;
    if(in!=null&&in.size()>0) ret=obtainRTIObjectClassIds(in);
    return ret;
  }// endi of getRTIObjectClassIds()

  /**
  * @return (objectClassName,objectClassId)
  */
  Hashtable obtainRTIObjectClassIds(Vector in) {
    int objClsId;
    String objClsName ;
    Hashtable ret = new Hashtable();
    if(in!=null) {
      Enumeration e = in.elements();
      if(e!=null)
      while(e.hasMoreElements()) {
        objClsName = (String)e.nextElement();
        if( objClsName!=null&& (objClsId=obtainRTIObjectClassIds(objClsName))>=0) 
            ret.put(objClsName,new Integer(objClsId));
      }// end of while
    }
    return ret;

  }// end of obtainRTIObjectClassIds() 

  /**
  * @return objectClassId
  */
  int obtainRTIObjectClassIds(String objClsName) {
    int objClsId=-1;
     if(objClsName!=null&&((objClsId=objectClassDB.getObjectClassId(objClsName))<0)) 
          objClsId=askRTIObjectClassId(objClsName);
    return objClsId;
  }// end of obtainRTIObjectClassIds() 

  /**
  * @return objectClassId
  */
  int askRTIObjectClassId(String objClsName) {
    int objClsId=-1;
    if( rtiAmb != null ) 
      try { objClsId = rtiAmb.getObjectClassHandle(objClsName); } 
      catch( RTIcap.FederateNotExecutionMember icnd ) {}
      catch( RTIcap.NameNotFound icnd ) {}
      catch( RTIcap.RTIinternalError icnd ) {}
    return objClsId;
  }// end of askRTIObjectClassId() 

  /**
  * @return (attributename,attributeid)
  */
  public Hashtable getRTIObjectClassAttributeIds(String objClassName, Vector in) {
    Hashtable ret = null;
    int objClsId=-1;
    if((objClsId=objectClassDB.getObjectClassId(objClassName))>=0&&in!=null&&in.size()>0) 
       ret = obtainRTIObjectClassAttributeIds(objClsId,in);
    return ret;
  }// end of getRTIObjectClassAttributeIds() 

  /**
  * @return (attributename,attributeid)
  */
  Hashtable obtainRTIObjectClassAttributeIds(int objClsId, Vector in) {
    int attId;
    String attName ;
    Hashtable ret = new Hashtable();
    Enumeration e = in.elements();
    while(e.hasMoreElements()) 
      if( (attName=(String)e.nextElement())!=null&&
          (attId=obtainRTIObjectClassAttributeId(objClsId,attName))>=0) 
           ret.put(attName,new Integer(attId));
    return ret;
  }// end of obtainRTIObjectClassAttributeIds() 

  /**
  * @return attributeid
  */
  int obtainRTIObjectClassAttributeId(int objClsId, String attName) {
    int attId=-1;
    if((attId=objectClassDB.getAttributeId(objClsId,attName))<0) 
        attId=askRTIObjectClassAttributeId(objClsId,attName);
    return attId;
  }// end of obtainRTIObjectClassAttributeId() 

  /**
  * @return attributeid
  */
  int askRTIObjectClassAttributeId(int objClsId, String attName) {
    int attId=-1;       
    if( rtiAmb!=null )
     try { attId = rtiAmb.getAttributeHandle(attName,(short)objClsId ); } 
     catch( RTIcap.ObjectClassNotDefined icnd ) {}
     catch( RTIcap.NameNotFound icnd ) {}
     catch( RTIcap.FederateNotExecutionMember icnd ) {}
     catch( RTIcap.RTIinternalError icnd ) {}
    return attId;
  }// end of askRTIObjectClassAttributeId() 

  /**
  * @return (interactionClassName,interactionClassId)
  */
  public Hashtable getRTIInteractionClassIds(Vector in) {
    Hashtable ret = null;
    if(in!=null&&in.size()>0) ret=obtainRTIInteractionClassIds(in);
    return ret;
  }// end of getRTIInteractionClassIds() 

  /**
  * @return (interactionClassName,interactionClassId)
  */
  Hashtable obtainRTIInteractionClassIds(Vector in) {
    int interClsId;
    String interClsName ;
    Hashtable ret = new Hashtable();
    Enumeration e = in.elements();
    while(e.hasMoreElements()) 
      if( (interClsName=(String)e.nextElement())!=null&&
          (interClsId=obtainRTIInteractionClassId(interClsName))>=0) 
           ret.put(interClsName,new Integer(interClsId));
    return ret;
  }// end of obtainRTIInteractionClassIds() 

  /**
  * @return interactionClassId
  */
  int obtainRTIInteractionClassId(String interClsName) {
    int interClsId=-1;
    if(((interClsId=interactionDB.getInteractionClassId(interClsName))<0)) 
         interClsId=askRTIInteractionClassId(interClsName);
    return interClsId;
  }// end of askRTIInteractionClassId() 

  /**
  * @return interactionClassId
  */
  int askRTIInteractionClassId(String interClsName) {
    int interClsId=-1;
    System.out.println("FederationDefinition.askRTIInteractionClassId(\""+interClsName+"\")");
    if( rtiAmb!=null )
      try { interClsId = rtiAmb.getInteractionClassHandle(interClsName); } 
      catch( RTIcap.NameNotFound icnd ) {}
      catch( RTIcap.FederateNotExecutionMember icnd ) {}
      catch( RTIcap.RTIinternalError icnd ) {}
    else System.out.println("FederationDefinition."+
                            "askRTIInteractionClassId() rtiAmb is NULL! ");
    return interClsId;
  }// end of askRTIInteractionClassId() 

  /**
  * @return (parameterName,parameterId)
  */
  public Hashtable getRTIParameterIds(String interactionClassName, Vector in) {
    int interClsId=-1;
    Hashtable ret = null;
    if( (interClsId=interactionDB.getInteractionClassId(interactionClassName))>=0 &&
        in!=null&&in.size()>0) 
        ret = obtainRTIParameterIds(interClsId, in);
    return ret;
  }// end of getRTIParameterIds() 

  /**
  * @return (parameterName,parameterId)
  */
  Hashtable obtainRTIParameterIds(int interClsId, Vector in) {
    int paramId;
    String paramName ;
    Hashtable ret = new Hashtable();
    Enumeration e = in.elements();
    while(e.hasMoreElements()) 
      if( (paramName=(String)e.nextElement())!=null &&
          (paramId=obtainRTIParameterId(interClsId,paramName))>=0 ) 
           ret.put(paramName,new Integer(paramId));
    return ret;
  }// end of obtainRTIParameterIds() 

  /**
  * @return parameterId
  */
  int obtainRTIParameterId(int interClsId, String paramName) {
    int paramId=-1;
    if( (paramId=interactionDB.getParameterId(interClsId,paramName))<0 ) 
        paramId=askRTIParameterId(interClsId,paramName);
    return paramId;
  }// end of askRTIParameterId() 

  /**
  * @return parameterId
  */
  int askRTIParameterId(int interClsId, String paramName) {
    int paramId=-1;
    System.out.println("FederationDefinition.askRTIParameterId("+interClsId+"\","+paramName+"\")");
    if( rtiAmb!=null )
     try { paramId = rtiAmb.getParameterHandle(paramName,(short)interClsId); } 
     catch( RTIcap.InteractionClassNotDefined icnd ) {}
     catch( RTIcap.NameNotFound icnd ) {}
     catch( RTIcap.FederateNotExecutionMember icnd ) {}
     catch( RTIcap.RTIinternalError icnd ) {} 
    return paramId;
  }// end of askRTIParameterId() 

// Begin --->>> RTIContextRegistry Interface 
  Vector rtiCnxs = new Vector();
  public void addRTIContext(RTIContext in) 
  { if(!rtiCnxs.contains(in)) rtiCnxs.addElement(in); }
  public void removeRTIContext(RTIContext in)
  { if(rtiCnxs.contains(in)) rtiCnxs.removeElement(in); }
// End of <<<-- RTIContextRegistry Interface 

  public void configure(RTIContext rticnx, JoinedFedInfo jfedinfo) {
    FedPubSubInfo fpubsub = jfedinfo.getFedPubSubInfo(); 
    // Set MessageFactory
    rticnx.setMessageFactory(federationName,msgFactory); 
    RTIMessagePusher rtiPusher = new RTIMessagePusher(rtiAmb);
    MessagePusher        mPush = new MessagePusher( discoverObjectMsg, 
                                                    updateAttributetMsg, 
                                                    updateParameterMsg,
                                                    rtiPusher);
    MessagePublisherImpl mPub  = new MessagePublisherImpl(mPush);
    rticnx.setMessagePublisher(federationName,mPub);
    addDisposer(rticnx,mPush); addDisposer(rticnx,mPub);
    addDisposer(rticnx,rtiPusher);
    // let's learn the RTI ids for the pub/sub
    Enumeration e = fpubsub.getPublishedObjects(); 
    if(e!=null&&e.hasMoreElements()) checkObjectRecs(e);
    e = fpubsub.getSubscribedObjects(); 
    if(e!=null&&e.hasMoreElements()) checkObjectRecs(e);
    e = fpubsub.getPublishedInteractions(); 
    if(e!=null&&e.hasMoreElements()) 
    { checkInteractionRecs(e); anouncePublishInteractionRecs(fpubsub.getPublishedInteractions()); }
    e = fpubsub.getSubscribedInteractions(); 
    if(e!=null&&e.hasMoreElements()) 
    { checkInteractionRecs(e); anounceSubscibeInteractionRecs(fpubsub.getSubscribedInteractions()); }
  }// end of configure()

  void checkObjectRecs(Enumeration e) {
    Vector attr;
    ObjRec oRec;
    String attrName;
    int    objClsId; 
    while( e.hasMoreElements() ) 
      if( (oRec=(ObjRec)e.nextElement())!=null&&oRec.getName()!=null&&
          (objClsId=obtainRTIObjectClassIds(oRec.getName()))>=0&&
          (attr=oRec.getAllAttributes())!=null&& attr.size()>0) 
            for(int i=0;i<attr.size();i++)  
              if((attrName=(String)attr.get(i))!=null) 
                  obtainRTIObjectClassAttributeId(objClsId,attrName);
  }// end of checkObjectRecs()

  void checkInteractionRecs(Enumeration e) {
    Vector   params;
    InterRec iRec;
    String   paramName;
    int      intClsId; 
    while( e.hasMoreElements() ) 
      if( (iRec=(InterRec)e.nextElement())!=null&&
          (intClsId=obtainRTIInteractionClassId(iRec.getName()))>=0&&
          (params=iRec.getAllAttributes())!=null&& params.size()>0) 
            for(int i=0;i<params.size();i++)  
              if((paramName=(String)params.get(i))!=null) 
                  obtainRTIParameterId(intClsId,paramName);
  }// end of checkObjectRecs()

  void publishInteractionClass(int intClsId) {
    if( rtiAmb!=null ) {
      System.out.println("FederationDefinition.publishInteractionClass("+intClsId+")");
      try { rtiAmb.publishInteractionClass((short)intClsId); } 
      catch( RTIcap.FederateNotExecutionMember fnem) {}
      catch( RTIcap.RestoreInProgress rip) {}
      catch( RTIcap.InteractionClassNotDefined icnd) {}
      catch( RTIcap.SaveInProgress sip) {}
      catch( RTIcap.RTIinternalError ie) {}
    }// end of if
  }// end of publishInteractionClass()

  void anouncePublishInteractionRecs(Enumeration e) {
    Vector   params;
    InterRec iRec;
    String   paramName;
    int      intClsId; 
    while( e.hasMoreElements() ) 
      if( (iRec=(InterRec)e.nextElement())!=null&&
          (intClsId=obtainRTIInteractionClassId(iRec.getName()))>=0)
          publishInteractionClass(intClsId);
  }// end of anouncePublishInteractionRecs()

  void subscribeInteractionClass(int intClsId) {
    if( rtiAmb!=null ) {
      System.out.println("FederationDefinition.subscribeInteractionClass("+intClsId+")");
     try { rtiAmb.subscribeInteractionClass((short)intClsId,RTIcap._Boolean.RTI_TRUE);}
     catch( RTIcap.FederateLoggingServiceCalls  flsc) {}
     catch( RTIcap.FederateNotExecutionMember  fnem) {}
     catch( RTIcap.RTIinternalError  ie) {}
     catch( RTIcap.RestoreInProgress  rip) {}
     catch( RTIcap.InteractionClassNotDefined  icnd) {}
     catch( RTIcap.SaveInProgress  sip) {}
   }// end of if
  }// end of subscribeInteractionClass()

  void anounceSubscibeInteractionRecs(Enumeration e) {
    Vector   params;
    InterRec iRec;
    String   paramName;
    int      intClsId; 
    while( e.hasMoreElements() ) 
      if( (iRec=(InterRec)e.nextElement())!=null&&
          (intClsId=obtainRTIInteractionClassId(iRec.getName()))>=0)
          subscribeInteractionClass(intClsId);
  }// end of anounceSubscibeInteractionRecs()

  // Now MessageHandler is set by the component
  public void postConfigure( RTIContext rticnx, CallbackRepository cbRep,
                             JoinedFedInfo jfedinfo ) {
    FedPubSubInfo fpubsub = jfedinfo.getFedPubSubInfo(); 
    PullProxyImpl p1 = new PullProxyImpl(rticnx.getRTIComponent(),cbRep,discoverObjectMsg,fpubsub);
    PullProxyImpl p2 = new PullProxyImpl(rticnx.getRTIComponent(),cbRep,updateAttributetMsg,fpubsub);
    PullProxyImpl p3 = new PullProxyImpl(rticnx.getRTIComponent(),cbRep,updateParameterMsg,fpubsub);
    addDisposer(rticnx,p1); addDisposer(rticnx,p2); addDisposer(rticnx,p3);
  }// end of postConfigure()

  Hashtable  disposer = new Hashtable();
  void addDisposer(RTIContext rticnx,Object ins) {
    Vector a = (Vector) disposer.get(rticnx);
    if(a==null) { a=new Vector(); disposer.put(rticnx,a); }
    a.addElement(ins);
  }// end of addDisposer()
  void dumpDisposer(RTIContext rticnx) {
    Vector a = (Vector) disposer.get(rticnx);
    Object curr;
    if(a!=null) {
     Enumeration e = a.elements();
     while(e.hasMoreElements()) 
       if( (curr=e.nextElement())!=null && curr instanceof Disposable)  
            ((Disposable)curr).dispose();
    }// end of if 
  }// end of addDisposer()

} // end of class FederationDefinition

