package xml;

import org.w3c.dom.*;
import java.util.*;
import java.io.*;


/**
*
* @date   July 26, 1999 ( Last Updated August 6, 1999 )
* @author H. Timucin Ozdemir
*
*/
public class FedPubSubInfo {
  Hashtable publishObjectMap = new Hashtable();
  Hashtable publishInteractionMap = new Hashtable();
  Hashtable subscribeObjectMap = new Hashtable();
  Hashtable subscribeInteractionMap = new Hashtable();
  String    fedName=null;
  public FedPubSubInfo(String _fedName) { fedName=_fedName; }
  public String getName() { return fedName; }
  public void addPublishObject(String _objClsName) {
    if(publishObjectMap.get(_objClsName)==null) 
       publishObjectMap.put(_objClsName,new ObjRec(_objClsName));
  }// end of addPublishObject()
  public void addPublishObjectAttribute(String _objClsName,String _attName) {
    ObjRec aRec = (ObjRec)publishObjectMap.get(_objClsName);
    if(aRec==null) { 
       addPublishObject(_objClsName); 
       aRec = (ObjRec)publishObjectMap.get(_objClsName);
    }// end of if
    if(aRec!=null) aRec.addAttribute(_attName);  
  }// end of addPublishObjectAttribute()
  public void addPublishInteraction(String _intClsName) {
    if(publishInteractionMap.get(_intClsName)==null) 
       publishInteractionMap.put(_intClsName,new InterRec(_intClsName));
  }// end of addPublishInteraction()
  public void addPublishParameter(String _intClsName,String _paramName) {
    InterRec aRec = (InterRec)publishInteractionMap.get(_intClsName);
    if(aRec==null) { 
       addPublishInteraction(_intClsName); 
       aRec = (InterRec)publishInteractionMap.get(_intClsName);
    }// end of if
    if(aRec!=null) aRec.addParameter(_paramName);  
  }// end of addPublishParameter()
  public void addSubscribeObject(String _objClsName) {
    if(subscribeObjectMap.get(_objClsName)==null) 
       subscribeObjectMap.put(_objClsName,new ObjRec(_objClsName));
  }// end of addSubscribeObject()
  public void addSubscribeObjectAttribute(String _objClsName,String _attName) {
    ObjRec aRec = (ObjRec)subscribeObjectMap.get(_objClsName);
    if(aRec==null) { 
       addSubscribeObject(_objClsName); 
       aRec = (ObjRec)subscribeObjectMap.get(_objClsName);
    }// end of if
    if(aRec!=null) aRec.addAttribute(_attName);  
  }// end of addSubscribeObjectAttribute()
  public void addSubscribeInteraction(String _intClsName) {
    if(subscribeInteractionMap.get(_intClsName)==null) 
       subscribeInteractionMap.put(_intClsName,new InterRec(_intClsName));
  }// end of addSubscribeInteraction
  public void addSubscribeParameter(String _intClsName,String _paramName) {
    InterRec aRec = (InterRec)subscribeInteractionMap.get(_intClsName);
    if(aRec==null) { 
       addSubscribeInteraction(_intClsName); 
       aRec = (InterRec)subscribeInteractionMap.get(_intClsName);
    }// end of if
    if(aRec!=null) aRec.addParameter(_paramName);  
  }// end of addSubscribeParameter()
  public Hashtable getPublishableObjectRecords() { return publishObjectMap; }
  public Hashtable getPublishableInteractionRecords() { return publishInteractionMap; }
  public Hashtable getSubscribedObjectRecords() { return subscribeObjectMap; }
  public Hashtable getSubscribedInteractionRecords() { return subscribeInteractionMap; }
  public boolean   canPublishAttribute(String _objClsName,String _attName) {
    ObjRec aRec = (ObjRec)publishObjectMap.get(_objClsName);
    return (aRec!=null&&aRec.contains(_attName));
  }
  public boolean   canPublishObjectType(String _objClsName) 
  { return (publishObjectMap.get(_objClsName)!=null); }
  public boolean   canConsumeAttribute(String _objClsName,String _attName) {
    ObjRec aRec = (ObjRec)subscribeObjectMap.get(_objClsName);
    return (aRec!=null&&_attName!=null&&aRec.contains(_attName));
  }
  public boolean   canConsumeObjectType(String _objClsName) 
  { return (subscribeObjectMap.get(_objClsName)!=null); }
  public boolean   canPublishParameter(String _intClsName,String _paramName) {
    InterRec aRec = (InterRec)publishInteractionMap.get(_intClsName);
    return (aRec!=null&&aRec.contains(_paramName));
  }
  public boolean   canPublishInteractionType(String _intClsName) 
  { return (publishInteractionMap.get(_intClsName)!=null); }
  public boolean   canConsumeParameter(String _intClsName,String _paramName) {
    InterRec aRec = (InterRec)subscribeInteractionMap.get(_intClsName);
    //System.out.println("FedPubSubInfo:: canConsumeParameter("+_intClsName+","+_paramName+") !"); 
    return (aRec!=null&&aRec.contains(_paramName));
  }
  public boolean   canConsumeInteractionType(String _intClsName) 
  { return (subscribeInteractionMap.get(_intClsName)!=null); }
  public Enumeration getPublishedObjects()       { return publishObjectMap.elements(); }
  public Enumeration getPublishedInteractions()  { return publishInteractionMap.elements(); }
  public Enumeration getSubscribedObjects()      { return subscribeObjectMap.elements(); }
  public Enumeration getSubscribedInteractions() { return subscribeInteractionMap.elements();}
  public void print(PrintStream o)  { 
    ObjRec oRec ; InterRec iRec ; String aKey;
    Enumeration e = publishObjectMap.keys();
    o.println(">Published Objects ");
    while( e.hasMoreElements() )
      if( (aKey = (String) e.nextElement())!=null&&
          (oRec=(ObjRec)publishObjectMap.get(aKey))!=null) oRec.print(o);
    o.println(">Published Interactions ");
    e=publishInteractionMap.keys();
    while( e.hasMoreElements() )
      if( (aKey = (String) e.nextElement())!=null&&
          (iRec=(InterRec)publishInteractionMap.get(aKey))!=null) iRec.print(o);
    o.println(">Subscribed Objects ");
    e = subscribeObjectMap.keys();
    while( e.hasMoreElements() )
      if( (aKey = (String) e.nextElement())!=null&&
          (oRec=(ObjRec)subscribeObjectMap.get(aKey))!=null) oRec.print(o);
    o.println(">Subscribed Interactions ");
    e=subscribeInteractionMap.keys();
    while( e.hasMoreElements() )
      if( (aKey = (String) e.nextElement())!=null&&
          (iRec=(InterRec)subscribeInteractionMap.get(aKey))!=null) iRec.print(o);
  }// end of print()
}// end of FedPubSubInfo
