package xml;

import org.w3c.dom.*;
import java.util.*;
import java.io.PrintStream;
import java.net.*;

import util.*;
import util.inmem.*;

/**
*
* @date   September 26, 1998 ( Last Updated September 26, 1998 )
* @author H. Timucin Ozdemir
*
*/
public class 
RTIComponentDocument extends BaseDocument implements XMLDebugFlags, RTIComponentConstants {
   
   Hashtable  cmaps;
   Properties props = new Properties();
   /** isExtracted? */
   boolean isExtracted = true;

   /** org.w3c.dom.Document */
   Document dom;

   String name=null;
   String className=null;
   String myPath=null;
   String myJarFileName=null;
   String myUrl=null;

   Hashtable feds   = new Hashtable();
   Hashtable cBacks = new Hashtable();

   /** */
   public RTIComponentDocument(Document _dom) {
             cmaps = new Hashtable();
               dom = _dom;
   }// end of CONSTRUCTOR

   /** */
   public Enumeration getRegisteredComponents() { 
      if(isExtracted) extractFromDocument();
      return cmaps.elements(); 
   }// end of getRegisteredComponents()

   /** */
   public Properties getProperties() { 
      if(isExtracted) extractFromDocument();
      return props;
   }// end of getProperties()

   /** */
   public String getComponentName() { 
      if(isExtracted) extractFromDocument();
      return name; 
   }// end of getComponentName()

   /** */
   public String getClassName() 
   { if(isExtracted) extractFromDocument(); return className; }

   /** */
   public String getPathInfo() 
   { if(isExtracted) extractFromDocument(); return myPath; }

   /** */
   public String getJarFileName() 
   { if(isExtracted) extractFromDocument(); return myJarFileName; }

   /** */
   public String getUrl() 
   { if(isExtracted) extractFromDocument(); return myUrl; }

   /** */
   public Hashtable getInvolvedFederations() 
   { if(isExtracted) extractFromDocument(); return feds; }

   /** */
   void addComponentRecord(ComponentRecord _item) { cmaps.put(_item.getComponentName(),_item);}

   /** */
   public void extractFromDocument() { 
      if(DEBUG) System.out.println(" in extractFromDocument() ");
      isExtracted = false;
      if(dom!=null) {
         Element componentsRoot = dom.getDocumentElement();
         if( componentsRoot.getTagName().equals(RTIComponentLabel) ) extractItems(componentsRoot);
      } else if(DEBUG) System.out.println(" Document is NULL1");
      if(DEBUG) System.out.println(" out of extractFromDocument() ");
   }// end of extractFromDocument()

   /** */
   final void extractItems(Element componentsRoot) {
     if(DEBUG) System.out.println(" in extractItems() ");
     String tName;
     NodeList componentList = ((Node)componentsRoot).getChildNodes();
     Node     current=null; 
     int      i=0;
     name=getAttributeValue(componentsRoot,nameLabel);
     if( componentList!=null && componentList.getLength()>0) {
        while( i<componentList.getLength() ) {
           current = componentList.item(i++);
           if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
             tName = ((Element)current).getTagName();
             if(tName.equals(propertyLabel)) extractProperties((Element)current);
             else if(tName.equals(implLabel)) extractImplementationDetails((Element)current);
             else if(tName.equals(fedLabel)) extractFederationDetails((Element)current);
           }// end of if
        }// end of while
     }// end of if 
     if(DEBUG) System.out.println(" out of extractItems() ");
   }// end of extractItems()

   /** */
   final void extractProperties(Element propertyItem) {
     if(DEBUG) System.out.println(" in extractProperties() ");
     Node     current=null; 
     NodeList bList = propertyItem.getChildNodes();
     if( bList!=null && bList.getLength()>0 ) {
       for(int i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          if( ((Element)current).getTagName().equals(attrLabel)) 
             props.put( getAttributeValue((Element)current,nameLabel),
                        getTextNodeValue((Element)current));
         }
       }// end of for
     }// end of if
     if(DEBUG) System.out.println(" out of extractProperties() ");
   }// end of extractProperties()

   /** */
   final void extractImplementationDetails(Element propertyItem) {
     if(DEBUG) System.out.println(" in extractImplementationDetails() ");
     String tName;
     Node     current=null; 
     NodeList bList = propertyItem.getChildNodes();
     if( bList!=null && bList.getLength()>0 ) {
       for(int i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          tName=((Element)current).getTagName();
          if(tName.equals(jarLabel)) myJarFileName=getTextNodeValue((Element)current);
          else if(tName.equals(clsfilelLabel)) className=getTextNodeValue((Element)current);
          else if(tName.equals(urlLabel)) myUrl=getTextNodeValue((Element)current);
          else if(tName.equals(pathLabel)) myPath=getTextNodeValue((Element)current);
         }
       }// end of for
     }// end of if
     if(DEBUG) System.out.println(" out of extractImplementationDetails() ");
   }// end of extractImplementationDetails()

   /** */
   final void extractFederationDetails(Element fedItem) {
     if(DEBUG) System.out.println(" in extractFederationDetails() ");
     String tName;
     Node     current=null; 
     NodeList bList = fedItem.getChildNodes();
     String fName = getAttributeValue(fedItem,nameLabel);
    
     if( fName!=null && bList!=null && bList.getLength()>0 ) {
       JoinedFedInfo fInfo = new JoinedFedInfo(fName);
       FedPubSubInfo fPubSubInfo = fInfo.getFedPubSubInfo();
       CallbackRegistry oCallbackRegistry = fInfo.getObjectCallbackRegistry();
       CallbackRegistry iCallbackRegistry = fInfo.getInteractionCallbackRegistry();
       feds.put(fName,fInfo);
       for(int i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          tName=((Element)current).getTagName();
          if(tName.equals(publishLabel)) 
               extractPubSubDetails((Element)current,fPubSubInfo,oCallbackRegistry,iCallbackRegistry,true);
          else if(tName.equals(subscribeLbl)) 
               extractPubSubDetails((Element)current,fPubSubInfo,oCallbackRegistry,iCallbackRegistry,false);
         }
       }// end of for
     }// end of if
     if(DEBUG) System.out.println(" out of extractFederationDetails() ");
   }// end of extractFederationDetails()

   /** */
   final void 
   extractPubSubDetails(Element anItem, FedPubSubInfo fPubSubInfo, 
                        CallbackRegistry oCallbackRegistry, CallbackRegistry iCallbackRegistry, boolean isPublish) {
     if(DEBUG) System.out.println(" in extractPubSubDetails() "+(isPublish?"Publish":"Subscibe"));
     String tName;
     Node     current=null; 
     NodeList bList = anItem.getChildNodes();
     if( bList!=null && bList.getLength()>0 ) {
       for(int i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          tName=((Element)current).getTagName();
          if(tName.equals(objectLabel)) 
               extractObjectInfo((Element)current,fPubSubInfo,oCallbackRegistry,isPublish);
          else if(tName.equals(interactionLbl)) 
               extractInteractionInfo((Element)current,fPubSubInfo,iCallbackRegistry,isPublish);
         }
       }// end of for
     }// end of if
     if(DEBUG) System.out.println(" out of extractPubSubDetails() "+(isPublish?"Publish":"Subscibe"));
   }// end of extractPubSubDetails()

   /** */
   final void 
   extractObjectInfo(Element anItem, FedPubSubInfo fpsing, 
                     CallbackRegistry oCallbackRegistry, boolean isPublish) {
     if(DEBUG) System.out.println(" in extractObjectInfo() ");
     String   tName, attrName, objName, callbackName;
     Node     current=null; 
     NodeList bList = anItem.getChildNodes();
     objName=getAttributeValue((Element)anItem,nameLabel); 
     if(DEBUG) System.out.println(" object name: "+objName);
     if( objName!=null && bList!=null && bList.getLength()>0 ) {
       for(int i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && 
             current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          tName=((Element)current).getTagName();
          if(tName.equals(attributeLabel)) {
             attrName=getAttributeValue((Element)current,nameLabel); 
             if(attrName!=null) {
              if(DEBUG) System.out.println(" attribute name: "+attrName);
              if(isPublish) fpsing.addPublishObject(objName);
              else fpsing.addSubscribeObject(objName);
              if(isPublish) fpsing.addPublishObjectAttribute(objName,attrName);
              else { fpsing.addSubscribeObjectAttribute(objName,attrName);
                if((callbackName=getAttributeValue((Element)current,callbackLabel))!=null)
                   oCallbackRegistry.addCallback(objName+"."+attrName,callbackName);
              }
             }// end of if
          }// end of if
         }// end of if
       }// end of for
     }// end of if
     if(DEBUG) System.out.println(" out of extractObjectInfo() ");
   }// end of extractObjectInfo()

   /** */
   final void 
   extractInteractionInfo(Element anItem, FedPubSubInfo fpsing, CallbackRegistry fCallbackRegistry, boolean isPublish) {
     if(DEBUG) System.out.println(" in extractInteractionInfo() ");
     String   tName, attrName, objName, callbackName;
     Node     current=null; 
     NodeList bList = anItem.getChildNodes();
     objName=getAttributeValue((Element)anItem,nameLabel);
     if(DEBUG) System.out.println(" interaction class name: "+objName);
     if( objName!=null && bList!=null && bList.getLength()>0 ) {
       for(int i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && 
             current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          tName=((Element)current).getTagName();
          if(tName.equals(paramLabel)) {
             attrName=getAttributeValue((Element)current,nameLabel); 
             if(attrName!=null) {
               if(DEBUG) System.out.println(" parameter name: "+attrName);
               if(isPublish) fpsing.addPublishInteraction(objName);
               else fpsing.addSubscribeInteraction(objName);
               if(isPublish) fpsing.addPublishParameter(objName,attrName);
               else {
                 fpsing.addSubscribeParameter(objName,attrName);
                 if((callbackName=getAttributeValue((Element)current,callbackLabel))!=null)
                    fCallbackRegistry.addCallback(objName+"."+attrName,callbackName);
               }// end of if
             }// end of if
          }// end of if
         }// end of if
       }// end of for
     }// end of if
     if(DEBUG) System.out.println(" out of extractInteractionInfo() ");
   }// end of extractInteractionInfo()

   /** */
   final void extractOneItem(Element componentItem) {
     if(DEBUG) System.out.println(" in extractOneItem() ");
     NodeList fedNames = ((Node)componentItem).getChildNodes();
     NodeList fedNames1 ;
     Node     current=null; 
     Node     current2=null; 
     int      i=0;
     String   cmp_name    = getAttributeValue(componentItem,nameLabel);
     String   cmp_imgfile = null;
     String   cmp_filename = null;
     String   cmp_filepath = null;
     String   cmp_fileurl  = null;
     fileNameRec a=null;
     NodeList bList = componentItem.getChildNodes();
     if( bList!=null && bList.getLength()>0 ) {
       for(i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          if( ((Element)current).getTagName().equals(imgLabel)) 
             cmp_imgfile = getAttributeValue((Element)current,fileLabel);
          if( ((Element)current).getTagName().equals(fileNameLabel)) 
             a=extractFileName((Element)current);
         }
       }// end of for
     }// end of if
     if(a!=null) {
       cmp_filename = a.getFileName();
       cmp_filepath = a.getFilePath();
       cmp_fileurl  = a.getFileUrl();
     }
     if(DEBUG) {
       System.out.println(" cmp_filename "+cmp_filename );
       System.out.println(" cmp_filepath "+cmp_filepath );
       System.out.println(" cmp_fileurl  "+cmp_fileurl  );
     }// end of if
     if( cmp_name!=null && cmp_filename!=null ) {
         ComponentRecord cmpItem = new ComponentRecord(cmp_name,cmp_imgfile,
                                                       cmp_filename,cmp_filepath,
                                                       cmp_fileurl);
         if(DEBUG) System.out.println(" add "+cmp_name);
         addComponentRecord(cmpItem);
        }// end  of if
     if(DEBUG) System.out.println(" out of extractOneItem() ");
   }// end of extractOneItem()

   /** */
   final fileNameRec extractFileName( Element componentItem) {
     String cmp_filename = null; 
     String cmp_filepath = null;
     String cmp_fileurl  = null;
     if(DEBUG) System.out.println(" in extractFileName() ");
     Node     current=null; 
     int      i=0;
     NodeList bList = componentItem.getChildNodes();
     if( bList!=null && bList.getLength()>0 ) {
       for(i=0;i<bList.getLength();i++) {
         current = bList.item(i);
         if( current!=null && current.getNodeType()==org.w3c.dom.Node.ELEMENT_NODE ) {
          String tg = ((Element)current).getTagName(); 
          if(tg.equals(nameLabel)) cmp_filename=getTextNodeValue((Element)current);
          else if(tg.equals(pathLabel)) cmp_filepath=getTextNodeValue((Element)current);
          else if(tg.equals(urlLabel))  cmp_fileurl=getTextNodeValue((Element)current);
         }
       }// end of for
     }// end of if
     if(DEBUG) {
       System.out.println(" cmp_filename "+cmp_filename );
       System.out.println(" cmp_filepath "+cmp_filepath );
       System.out.println(" cmp_fileurl  "+cmp_fileurl  );
       System.out.println(" out of extractFileName() ");
     }// end of if
     return new fileNameRec(cmp_filename, cmp_filepath, cmp_fileurl);
   }// end of extractFileName()

   /** */
   public void print(PrintStream o)  { 
      o.println( "<"+RTIComponentLabel+">");
      Enumeration e = cmaps.elements();
      while( e.hasMoreElements() ) {
         ComponentRecord imItem = (ComponentRecord) e.nextElement();
         if( imItem!=null ) imItem.print(o);
      }// end of while
      o.println( "</"+RTIComponentLabel+">");
   }// end of print()

   /** */
   public void printSummary(PrintStream o)  { 
     o.println( ">>>> Summary <<<<<");
     Enumeration e = feds.elements();
     JoinedFedInfo imItem ;
     while( e.hasMoreElements() )         
       if( (imItem=(JoinedFedInfo)e.nextElement())!=null ) imItem.print(o);
     o.println( "<<<< Summary >>>>>");
   }// end of print()

   class fileNameRec {
     String cmp_filename, cmp_filepath, cmp_fileurl;
     fileNameRec( String _cmp_filename, String _cmp_filepath, String _cmp_fileurl) {
       cmp_filename=_cmp_filename; cmp_filepath=_cmp_filepath; cmp_fileurl=_cmp_fileurl;
     } 
     public String getFileName() { return cmp_filename; }
     public String getFilePath() { return cmp_filepath; }
     public String getFileUrl()  { return cmp_fileurl; }
   }

}// end of RTIComponentDocument
